package main

import (
	"bufio"
	"encoding/base64"
	"encoding/json"
	"fmt"
	"github.com/pion/webrtc/v2"
	"os"
//	"time"
)

func check(e error) {
	if e != nil {
		panic(e)
	}
}

func main() {

	m := webrtc.MediaEngine{}
	m.RegisterCodec(webrtc.NewRTPOpusCodec(webrtc.DefaultPayloadTypeOpus, 48000))
	se := webrtc.SettingEngine{}
	se.DetachDataChannels()
	api := webrtc.NewAPI(webrtc.WithMediaEngine(m), webrtc.WithSettingEngine(se))
	pc, err := api.NewPeerConnection(webrtc.Configuration{})
	check(err)

	pc.OnICEConnectionStateChange(func(connectionState webrtc.ICEConnectionState) {
		fmt.Printf("client: ICE state changed: %s\n", connectionState.String())
	})

/*	pc.OnDataChannel(func(dc *webrtc.DataChannel) {
		fmt.Println("ondatachannel")
		dc.OnMessage(func(msg webrtc.DataChannelMessage) {
			fmt.Println("onmessage", msg)
		})
	})
*/
	//dc, err := pc.CreateDataChannel("dc", nil)
	ordered := false
	maxrtx := uint16(0)
	opt := &webrtc.DataChannelInit{
		Ordered: &ordered,
		MaxRetransmits: &maxrtx,
	}
	dc, err := pc.CreateDataChannel("dc", opt)
	check(err)
	dc.OnOpen(func() {
		fmt.Printf("dc.onopen\n")
/*
		ticker := time.NewTicker(20 * time.Millisecond)
		defer ticker.Stop()
		for i:=0; ; i++ {
			select {
			case <-ticker.C:
				buf := make([]byte, 1500)
				err := dc.Send(buf)
				check(err)
				fmt.Printf("tx %d\n", i)
			}
		}
*/
		raw, err := dc.Detach()
		check(err)
		buf := make([]byte, 3000)
		for i:=0; ; i++ {
			_, err := raw.Read(buf)
			check(err)
			fmt.Printf("rx %d\n", i)
		}
	})
	dc.OnClose(func() {
		fmt.Printf("dc.onclose\n")
	})
	offer, err := pc.CreateOffer(nil)
	check(err)
	pc.SetLocalDescription(offer)
	desc, err := json.Marshal(offer)
	j := base64.StdEncoding.EncodeToString(desc)
	fmt.Println("offer =", j)
	fmt.Println("answer?")
	r := bufio.NewReader(os.Stdin)
	s, err := r.ReadString('\n')
	check(err)
	answer, err := base64.StdEncoding.DecodeString(s)
	check(err)
	var ansdesc webrtc.SessionDescription
	json.Unmarshal([]byte(answer), &ansdesc)
	err = pc.SetRemoteDescription(ansdesc)
	check(err)

	select {}
}
